@@ -9,23 +9,22 @@ import pytest
 from hud.datasets.utils import (
     BatchRequest,
     SingleTaskRequest,
-    calculate_group_stats,
     cancel_all_jobs,
     cancel_job,
     cancel_task,
-    display_results,
     submit_rollouts,
 )
-from hud.types import AgentType, Task, Trace
+from hud.eval.display import display_results
+from hud.types import AgentType, LegacyTask, Trace
 
 
 class TestSingleTaskRequest:
     """Tests for SingleTaskRequest schema."""
 
     def test_valid_request(self):
-        """Test creating a valid SingleTaskRequest."""
+        """Test creating a valid SingleTaskRequest with v5 task."""
         request = SingleTaskRequest(
-            task={"prompt": "test", "mcp_config": {}},
+            task={"env": {"name": "browser"}, "scenario": "checkout"},
             agent_type=AgentType.CLAUDE,
             agent_params={"checkpoint_name": "claude-sonnet-4-5"},
             max_steps=10,
@@ -48,8 +47,8 @@ class TestSingleTaskRequest:
             )
 
     def test_invalid_task_rejected(self):
-        """Test that invalid task payload is rejected."""
-        with pytest.raises(ValueError, match="Invalid task payload"):
+        """Test that invalid task payload is rejected (neither v4 nor v5)."""
+        with pytest.raises(ValueError, match="Task must have 'env'"):
             SingleTaskRequest(
                 task={"invalid_field": "test"},  # Missing required fields
                 agent_type=AgentType.CLAUDE,
@@ -58,6 +57,49 @@ class TestSingleTaskRequest:
                 trace_name="Test",
             )
 
+    def test_incomplete_v4_task_rejected(self):
+        """Test that incomplete v4 task (missing evaluate_tool) is rejected."""
+        # When prompt + mcp_config is present but evaluate_tool is missing,
+        # it's detected as v4 format but fails validation
+        with pytest.raises(ValueError, match="v4 task missing required fields"):
+            SingleTaskRequest(
+                task={
+                    "prompt": "test",
+                    "mcp_config": {"server": {"url": "http://localhost"}},
+                    # Missing evaluate_tool
+                },
+                agent_type=AgentType.CLAUDE,
+                job_id="job-123",
+                task_id="task-1",
+                trace_name="Test",
+            )
+
+    def test_valid_v4_task_accepted(self):
+        """Test that complete v4 task is accepted."""
+        request = SingleTaskRequest(
+            task={
+                "prompt": "test",
+                "mcp_config": {"server": {"url": "http://localhost"}},
+                "evaluate_tool": {"name": "check", "arguments": {}},
+            },
+            agent_type=AgentType.CLAUDE,
+            job_id="job-123",
+            task_id="task-1",
+            trace_name="Test",
+        )
+        assert request.task_id == "task-1"
+
+    def test_valid_v5_task_accepted(self):
+        """Test that v5 task with env is accepted."""
+        request = SingleTaskRequest(
+            task={"env": {"name": "browser"}, "scenario": "login"},
+            agent_type=AgentType.CLAUDE,
+            job_id="job-123",
+            task_id="task-1",
+            trace_name="Test",
+        )
+        assert request.task_id == "task-1"
+
 
 class TestBatchRequest:
     """Tests for BatchRequest schema."""
@@ -66,7 +108,7 @@ class TestBatchRequest:
         """Test creating a valid batch request."""
         requests = [
             SingleTaskRequest(
-                task={"prompt": "test", "mcp_config": {}},
+                task={"env": {"name": "browser"}, "scenario": "test"},
                 agent_type=AgentType.CLAUDE,
                 job_id="job-123",
                 task_id=f"task-{i}",
@@ -155,64 +197,14 @@ class TestCancellationFunctions:
             assert result["total_tasks_cancelled"] == 10
 
 
-class TestCalculateGroupStats:
-    """Tests for calculate_group_stats function."""
-
-    def test_basic_stats(self):
-        """Test basic group statistics calculation."""
-        tasks = [
-            Task(prompt="Task 1", mcp_config={}),
-            Task(prompt="Task 2", mcp_config={}),
-        ]
-        traces: list[Trace | None] = [
-            Trace(reward=0.8, done=True),
-            Trace(reward=0.9, done=True),
-            Trace(reward=0.6, done=True),
-            Trace(reward=0.7, done=True),
-        ]
-        group_ids = {0: "group-0", 1: "group-1"}
-
-        stats = calculate_group_stats(tasks, traces, group_size=2, group_ids=group_ids)
-
-        assert len(stats) == 2
-        assert stats[0]["mean_reward"] == pytest.approx(0.85, rel=0.01)
-        assert stats[1]["mean_reward"] == pytest.approx(0.65, rel=0.01)
-
-    def test_all_none_traces(self):
-        """Test when all traces are None."""
-        tasks = [Task(prompt="Task 1", mcp_config={})]
-        traces: list[Trace | None] = [None, None]
-        group_ids = {0: "group-0"}
-
-        stats = calculate_group_stats(tasks, traces, group_size=2, group_ids=group_ids)
-
-        assert len(stats) == 1
-        assert stats[0]["error_rate"] == 1.0
-        assert stats[0]["mean_reward"] == 0.0
-
-    def test_mixed_success_failure(self):
-        """Test with mixed success and failure traces."""
-        tasks = [Task(prompt="Task 1", mcp_config={})]
-        traces: list[Trace | None] = [
-            Trace(reward=1.0, done=True),
-            Trace(reward=0.0, done=True, isError=True),
-        ]
-        group_ids = {0: "group-0"}
-
-        stats = calculate_group_stats(tasks, traces, group_size=2, group_ids=group_ids)
-
-        assert stats[0]["success_rate"] == 0.5
-        assert stats[0]["error_rate"] == 0.5
-
-
 class TestDisplayResults:
     """Tests for display_results function."""
 
     def test_display_with_traces(self):
         """Test displaying single-run trace results."""
         tasks = [
-            Task(id="t1", prompt="Test task 1", mcp_config={}),
-            Task(id="t2", prompt="Test task 2", mcp_config={}),
+            LegacyTask(id="t1", prompt="Test task 1", mcp_config={}),
+            LegacyTask(id="t2", prompt="Test task 2", mcp_config={}),
         ]
         results = [
             Trace(reward=0.9, done=True),
@@ -225,7 +217,7 @@ class TestDisplayResults:
     def test_display_with_group_stats(self):
         """Test displaying group statistics."""
         tasks = [
-            Task(id="t1", prompt="Test task 1", mcp_config={}),
+            LegacyTask(id="t1", prompt="Test task 1", mcp_config={}),
         ]
         results = [
             {
@@ -246,7 +238,7 @@ class TestDisplayResults:
 
     def test_display_empty_results(self):
         """Test displaying when no valid results."""
-        tasks = [Task(prompt="Test", mcp_config={})]
+        tasks = [LegacyTask(prompt="Test", mcp_config={})]
         results: list[Trace | None] = [None]
 
         # Should not raise
@@ -258,8 +250,10 @@ class TestSubmitRollouts:
 
     @pytest.mark.asyncio
     async def test_submit_single_task(self):
-        """Test submitting a single task."""
-        tasks = [Task(id="task-1", prompt="Test prompt", mcp_config={})]
+        """Test submitting a single task (v5 format)."""
+        from hud.eval.task import Task
+
+        tasks = [Task(env={"name": "browser"}, scenario="test", id="task-1")]
 
         with patch("hud.datasets.utils.httpx.AsyncClient") as mock_client_cls:
             mock_response = MagicMock()
@@ -288,7 +282,9 @@ class TestSubmitRollouts:
     @pytest.mark.asyncio
     async def test_submit_with_group_size(self):
         """Test submitting with group_size > 1 creates multiple requests per task."""
-        tasks = [Task(id="task-1", prompt="Test prompt", mcp_config={})]
+        from hud.eval.task import Task
+
+        tasks = [Task(env={"name": "browser"}, scenario="test", id="task-1")]
 
         with patch("hud.datasets.utils.httpx.AsyncClient") as mock_client_cls:
             mock_response = MagicMock()
