@@ -31,50 +31,108 @@ class AgentType(str, Enum):
 
     @property
     def cls(self) -> type:
-        from hud.agents import ClaudeAgent, GeminiAgent, OpenAIAgent, OperatorAgent
-        from hud.agents.gemini_cua import GeminiCUAAgent
-        from hud.agents.openai_chat import OpenAIChatAgent
+        if self == AgentType.CLAUDE:
+            from hud.agents.claude import ClaudeAgent
 
-        mapping: dict[AgentType, type] = {
-            AgentType.CLAUDE: ClaudeAgent,
-            AgentType.OPENAI: OpenAIAgent,
-            AgentType.OPERATOR: OperatorAgent,
-            AgentType.GEMINI: GeminiAgent,
-            AgentType.GEMINI_CUA: GeminiCUAAgent,
-            AgentType.OPENAI_COMPATIBLE: OpenAIChatAgent,
-        }
-        if self == AgentType.INTEGRATION_TEST:
+            return ClaudeAgent
+        elif self == AgentType.OPENAI:
+            from hud.agents import OpenAIAgent
+
+            return OpenAIAgent
+        elif self == AgentType.OPERATOR:
+            from hud.agents import OperatorAgent
+
+            return OperatorAgent
+        elif self == AgentType.GEMINI:
+            from hud.agents.gemini import GeminiAgent
+
+            return GeminiAgent
+        elif self == AgentType.GEMINI_CUA:
+            from hud.agents.gemini_cua import GeminiCUAAgent
+
+            return GeminiCUAAgent
+        elif self == AgentType.OPENAI_COMPATIBLE:
+            from hud.agents.openai_chat import OpenAIChatAgent
+
+            return OpenAIChatAgent
+        elif self == AgentType.INTEGRATION_TEST:
             from hud.agents.misc.integration_test_agent import IntegrationTestRunner
 
             return IntegrationTestRunner
-        if self not in mapping:
+        else:
             raise ValueError(f"Unsupported agent type: {self}")
+
+    @property
+    def config_cls(self) -> type:
+        """Get config class without importing agent (avoids SDK dependency)."""
+        from hud.agents.types import (
+            ClaudeConfig,
+            GeminiConfig,
+            GeminiCUAConfig,
+            OpenAIChatConfig,
+            OpenAIConfig,
+            OperatorConfig,
+        )
+
+        mapping: dict[AgentType, type] = {
+            AgentType.CLAUDE: ClaudeConfig,
+            AgentType.OPENAI: OpenAIConfig,
+            AgentType.OPERATOR: OperatorConfig,
+            AgentType.GEMINI: GeminiConfig,
+            AgentType.GEMINI_CUA: GeminiCUAConfig,
+            AgentType.OPENAI_COMPATIBLE: OpenAIChatConfig,
+            AgentType.INTEGRATION_TEST: BaseAgentConfig,
+        }
+        if self not in mapping:
+            raise ValueError(f"Unsupported agent type for config: {self}")
         return mapping[self]
 
 
 class BaseAgentConfig(BaseModel):
-    """Standard agent configuration that tasks can override.
-    Provider-specific configs should not be included here.
+    """Agent configuration for LLM-specific settings.
+
+    Note: allowed_tools, disallowed_tools, response_tool_name, append_setup_output,
+    and initial_screenshot are kept for backwards compatibility with v4 task configs
+    but are no longer applied at the agent level. These should be configured on the
+    Environment/Task instead.
     """
 
-    model_config = ConfigDict(arbitrary_types_allowed=True, extra="forbid")
+    model_config = ConfigDict(arbitrary_types_allowed=True, extra="forbid", populate_by_name=True)
 
+    # LLM-specific setting
+    system_prompt: str | None = None
+
+    # Deprecated: kept for backwards compat with v4 task configs, not applied by agent
     allowed_tools: list[str] | None = None
     disallowed_tools: list[str] | None = None
     response_tool_name: str | None = None
-    system_prompt: str | None = None
     append_setup_output: bool = True
+    append_setup_tool: bool = True  # Alias for append_setup_output (backwards compat)
     initial_screenshot: bool = True
 
 
-class Task(BaseModel):
+class LegacyTask(BaseModel):
     """
+    DEPRECATED: Use Task from env() instead.
+
     A task configuration that can be used to create a task.
 
     The mcp_config field supports environment variable substitution using
     template placeholders in the format ${VAR_NAME} or ${VAR_NAME:default_value}.
 
-    Example:
+    .. deprecated:: 0.5.0
+        LegacyTask is deprecated in v0.5.0 and will be removed in v0.6.0
+        (no earlier than March 1st, 2026).
+
+        Use one of these migration paths:
+
+        1. Quick conversion: ``Task.from_v4(legacy_task)`` converts LegacyTask to Task
+        2. Full migration: Use ``@env.scenario()`` with setup code before first yield
+           and evaluate code after first yield
+
+        See https://docs.hud.ai/migration for the full migration guide.
+
+    Example (deprecated):
         mcp_config: {
             "hud": {
                 "url": "${HUD_MCP_URL:https://mcp.hud.ai/v3/mcp}",
@@ -95,6 +153,20 @@ class Task(BaseModel):
     agent_config: BaseAgentConfig | None = None
     metadata: dict[str, Any] = Field(default_factory=dict)
 
+    def __init__(self, **data: Any) -> None:
+        """Initialize LegacyTask with deprecation warning."""
+        import warnings
+
+        warnings.warn(
+            "LegacyTask is deprecated in v0.5.0 and will be removed in v0.6.0 "
+            "(no earlier than March 1st, 2026). "
+            "Use Task.from_v4() for quick conversion, or migrate to @env.scenario(). "
+            "See https://docs.hud.ai/migration for details.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        super().__init__(**data)
+
     @field_validator("mcp_config", "metadata", mode="before")
     @classmethod
     def parse_json_strings(cls, v: Any) -> Any:
@@ -208,7 +280,9 @@ class MCPToolCall(CallToolRequestParams):
 
 
 class MCPToolResult(CallToolResult):
-    """A tool result."""
+    """A tool result with optional call_id for correlation."""
+
+    call_id: str | None = None  # For correlating with provider-specific tool call IDs
 
     def _get_content_summary(self) -> str:
         """Extract a summary of the content."""
@@ -311,6 +385,27 @@ class TraceStep(BaseModel):
     model_config = ConfigDict(populate_by_name=True, extra="allow")
 
 
+class HudSpan(BaseModel):
+    """A telemetry span ready for export to HUD API."""
+
+    name: str
+    trace_id: str = Field(pattern=r"^[0-9a-fA-F]{32}$")
+    span_id: str = Field(pattern=r"^[0-9a-fA-F]{16}$")
+    parent_span_id: str | None = Field(default=None, pattern=r"^[0-9a-fA-F]{16}$")
+
+    start_time: str  # ISO format
+    end_time: str  # ISO format
+
+    status_code: str  # "UNSET", "OK", "ERROR"
+    status_message: str | None = None
+
+    attributes: TraceStep
+    exceptions: list[dict[str, Any]] | None = None
+    internal_type: str | None = None
+
+    model_config = ConfigDict(extra="forbid")
+
+
 class Trace(BaseModel):
     """Unified result from agent execution (task or prompt).
 
@@ -330,7 +425,7 @@ class Trace(BaseModel):
     isError: bool = Field(default=False)
 
     # Metadata
-    task: Task | None = Field(default=None)
+    task: LegacyTask | None = Field(default=None)
 
     # Trace
     trace: list[TraceStep] = Field(default_factory=list)
@@ -346,27 +441,22 @@ class Trace(BaseModel):
     def append(self, step: TraceStep) -> None:
         self.trace.append(step)
 
-    def populate_from_context(self) -> None:
-        """Populate trace steps from the current trace context if available.
 
-        This checks if we're executing within a hud.trace() context and
-        automatically populates the trace field with collected steps.
-        """
-        from hud.otel.context import get_current_task_run_id
-        from hud.telemetry.replay import get_trace
-
-        task_run_id = get_current_task_run_id()
-        if task_run_id:
-            collected_trace = get_trace(task_run_id)
-            if collected_trace:
-                self.trace = collected_trace.trace
+# Re-export Task for backwards compatibility (after module defs to avoid circular import)
+from hud.eval.task import Task  # noqa: E402
 
+# Type alias for functions that accept v5 Task, v4 LegacyTask, or raw dicts
+TaskInput = Task | LegacyTask | dict[str, Any]
 
 __all__ = [
     "AgentResponse",
     "AgentType",
+    "HudSpan",
+    "LegacyTask",
     "MCPToolCall",
     "MCPToolResult",
+    "Task",
+    "TaskInput",
     "Trace",
     "TraceStep",
 ]
